"""
web2ldap plugin classes for Active Directory (see draft-armijo-ldap-syntax)

$Id: activedirectory.py,v 1.8 2005/02/06 03:01:27 michael Exp $
"""

import time,utctime

from w2lapp.gui import DisplayDN

from w2lapp.schema.syntaxes import \
  LDAPSyntax,Boolean,DistinguishedName,DirectoryString,GeneralizedTime, \
  Integer,OctetString,SelectList,syntax_registry

class SAMAccountType(SelectList):
  """
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_samaccounttype.asp
  """
  oid = 'SAMAccountType-oid'
  desc = 'SAM-Account-Type in MS AD'
  attr_value_dict = {
     '268435456':'SAM_GROUP_OBJECT',
     '268435457':'SAM_NON_SECURITY_GROUP_OBJECT',
     '536870912':'SAM_ALIAS_OBJECT',
     '536870913':'SAM_NON_SECURITY_ALIAS_OBJECT',
     '805306368':'SAM_NORMAL_USER_ACCOUNT',
     '805306369':'SAM_MACHINE_ACCOUNT',
     '805306370':'SAM_TRUST_ACCOUNT',
    '1073741824':'SAM_APP_BASIC_GROUP',
    '1073741825':'SAM_APP_QUERY_GROUP',
    '2147483647':'SAM_ACCOUNT_TYPE_MAX',
  }

# Register certain attribute types for syntax classes
syntax_registry.registerAttrType(
  SAMAccountType.oid,[
    '1.2.840.113556.1.4.302', # sAMAccountType
  ]
)


class BitArrayInteger(Integer):
  oid = 'BitArrayInteger-oid'
  flag_desc_table = ()

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    attrValue_int = int(self.attrValue)
    return """%s<br>
    <table summary="Flags">
    <tr><th>Property flag</th><th>Value</th><th>Status</th></tr>
    %s
    </table>
    """ % (
      Integer.displayValue(self,sid,form,ls,dn,valueindex,commandbutton),
      '\n'.join([
        '<tr><td>%s</td><td>%s</td><td>%s</td></tr>' % (
          desc,
          hex(flag_value),
          {0:'-',1:'on'}[int((attrValue_int & flag_value)>0)]
        )
        for desc,flag_value in self.flag_desc_table
      ])
    )
  

class UserAccountControl(BitArrayInteger):
  """
  See knowledge base article 305144:
  http://support.microsoft.com/default.aspx?scid=kb;en-us;Q305144
  """
  oid = 'UserAccountControl-oid'
  flag_desc_table = (
    ('SCRIPT',0x0001),
    ('ACCOUNTDISABLE',0x0002),
    ('HOMEDIR_REQUIRED',0x0008),
    ('LOCKOUT',0x0010),
    ('PASSWD_NOTREQD',0x0020),
    ('PASSWD_CANT_CHANGE',0x0040),
    ('ENCRYPTED_TEXT_PWD_ALLOWED',0x0080),
    ('TEMP_DUPLICATE_ACCOUNT',0x0100),
    ('NORMAL_ACCOUNT',0x0200),
    ('INTERDOMAIN_TRUST_ACCOUNT',0x0800),
    ('WORKSTATION_TRUST_ACCOUNT',0x1000),
    ('SERVER_TRUST_ACCOUNT',0x2000),
    ('DONT_EXPIRE_PASSWORD',0x10000),
    ('MNS_LOGON_ACCOUNT',0x20000),
    ('SMARTCARD_REQUIRED',0x40000),
    ('TRUSTED_FOR_DELEGATION',0x80000),
    ('NOT_DELEGATED',0x100000),
    ('USE_DES_KEY_ONLY',0x200000),
    ('DONT_REQ_PREAUTH',0x400000),
    ('PASSWORD_EXPIRED',0x800000),
    ('TRUSTED_TO_AUTH_FOR_DELEGATION',0x1000000)
  )

syntax_registry.registerAttrType(
  UserAccountControl.oid,[
    '1.2.840.113556.1.4.8', # userAccountControl
  ]
)


class InstanceType(BitArrayInteger):
  """
  http://msdn.microsoft.com/library/en-us/adschema/adschema/a_instancetype.asp
  """
  oid = 'InstanceType-oid'
  flag_desc_table = (
    ('The head of naming context.',0x00000001),
    ('This replica is not instantiated.',0x00000002),
    ('The object is writable on this directory.',0x00000004),
    ('The naming context above this one on this directory is held.',0x00000008),
    ('The naming context is in the process of being constructed for the first time via replication.',0x00000010),
    ('The naming context is in the process of being removed from the local DSA.',0x00000020),
  )

syntax_registry.registerAttrType(
  InstanceType.oid,[
    '1.2.840.113556.1.2.1', # instanceType
  ]
)


class DNWithOctetString(DistinguishedName):
  oid = '1.2.840.113556.1.4.903'
  desc = 'DNWithOctetString'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    octet_tag,count,octet_string,dn = self.attrValue.split(':')
    return ':'.join([
      octet_tag,count,octet_string,
      DisplayDN(sid,form,ls,dn,commandbutton=commandbutton)
    ])


class DNWithString(DNWithOctetString):
  oid = '1.2.840.113556.1.4.904'
  desc = 'DNWithString'


class MicrosoftLargeInteger(Integer):
  oid = '1.2.840.113556.1.4.906'
  desc = 'Integer guaranteed to support 64 bit numbers'


class ObjectSecurityDescriptor(OctetString):
  oid = '1.2.840.113556.1.4.907'
  desc = 'Object-Security-Descriptor'


class Interval(MicrosoftLargeInteger):
  oid = 'Interval-oid'
  desc = 'Large integer with timestamp expressed as 100 nanoseconds since 1601-01-01 00:00'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    if self.attrValue=='9223372036854775807':
      return '-1: unlimited/off'
    else:
      delta = (long(self.attrValue)-116444736000000000L)/10000000
      if delta>=0:
        return utctime.strftimeiso8601(time.gmtime(delta)).encode('ascii')
      else:
        return self.attrValue.encode('ascii')

# Register certain attribute types for Interval
syntax_registry.registerAttrType(
  Interval.oid,[
    '1.2.840.113556.1.4.159',  # accountExpires
    '1.2.840.113556.1.4.49',   # badPasswordTime
    '1.2.840.113556.1.4.52',   # lastLogon
    '1.2.840.113556.1.4.1696', # lastLogonTimestamp
    '1.2.840.113556.1.4.51',   # lastLogoff
    '1.2.840.113556.1.4.96',   # pwdLastSet
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))


# Work around for Active Directory of Windows 2000:
# Register syntaxes by odd names
syntax_registry.oid2syntax['Boolean'] = Boolean
syntax_registry.oid2syntax['DN'] = DistinguishedName
syntax_registry.oid2syntax['Integer'] = Integer
syntax_registry.oid2syntax['DirectoryString'] = DirectoryString
syntax_registry.oid2syntax['GeneralizedTime'] = GeneralizedTime

syntax_registry.registerAttrType(
  DistinguishedName.oid,[
    'configurationNamingContext',
    'defaultNamingContext',
    'dsServiceName',
    'rootDomainNamingContext',
    'schemaNamingContext',
  ]
)

